﻿#region using

using System;
using System.Web;
using System.Web.UI;
using BlogEngine.Core.Web.Controls;
using BlogEngine.Core;
using System.Collections.Generic;
using System.IO;
using System.Xml;

#endregion

/// <summary>
/// A Post View Counter.
/// </summary>
[Extension("统计文章的点击次数", "1.0", "<a href=\"http://www.blogyi.net\">BlogYi.NET</a>",1011)]
public class PostViewCounter
{
    private static string dataSource = System.Web.HttpContext.Current.Server.MapPath(BlogSettings.Instance.StorageLocation) + "postviewcounts.xml";

    public PostViewCounter()
    {
        Post.Serving += new EventHandler<ServingEventArgs>(Post_Serving);
    }

    /// <summary>
    /// Handles the Post.Serving event to display post counts when logged in.
    /// </summary>
    private void Post_Serving(object sender, ServingEventArgs e)
    {
        Post myPost = (Post)sender;

        if (e.Location == ServingLocation.SinglePost || e.Location == ServingLocation.Feed)
        {
            List<PostInfo> posts = new List<PostInfo>();
            bool postFound = false;

            // Read it in
            if (File.Exists(dataSource))
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(dataSource);

                foreach (XmlNode node in doc.SelectNodes("posts/post"))
                {
                    int count = int.Parse(node.Attributes["count"].InnerText);
                    string id = node.InnerText;

                    PostInfo temp = new PostInfo(id, count);

                    if (temp.PostID == myPost.Id.ToString())
                    {
                        // Increment the total and don't add filename to list
                        temp.Count++;
                        postFound = true;
                    }

                    posts.Add(temp);
                }
            }

            // Add new file in if not in file
            // Also covers case where file does not exist.
            if (!postFound)
            {
                PostInfo temp = new PostInfo(myPost.Id.ToString(), 1);
                posts.Add(temp);
            }

            // Write it out
            using (XmlTextWriter writer = new XmlTextWriter(dataSource, System.Text.Encoding.UTF8))
            {
                writer.Formatting = Formatting.Indented;
                writer.Indentation = 4;
                writer.WriteStartDocument(true);
                writer.WriteStartElement("posts");

                foreach (PostInfo post in posts)
                {
                    writer.WriteStartElement("post");
                    writer.WriteAttributeString("count", post.Count.ToString());
                    writer.WriteValue(post.PostID);
                    writer.WriteEndElement();
                }

                writer.WriteEndElement();
            }
        }

        if (System.Threading.Thread.CurrentPrincipal.Identity.IsAuthenticated)
        {
            int count = GetPostCount(myPost.Id.ToString());

            e.Body += string.Format("<span class=\"postviewcount\"> - 点击:{0}次</span>", count);
        }
    }

    /// <summary>
    /// Retrieve view count from storage
    /// </summary>
    /// <param name="fileName">ID of post to look up</param>
    /// <returns></returns>
    private int GetPostCount(string postID)
    {
        int postCount = 0;

        if (File.Exists(dataSource))
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(dataSource);

            foreach (XmlNode node in doc.SelectNodes("posts/post"))
            {
                int count = int.Parse(node.Attributes["count"].InnerText);
                string name = node.InnerText;

                if (name == postID)
                {
                    postCount = count;
                    break;
                }
            }
        }

        return postCount;
    }

    /// <summary>
    /// Simple class to hold Post Simple info
    /// </summary>
    private class PostInfo
    {
        public PostInfo(string postID, int count)
        {
            _postID = postID;
            _count = count;
        }

        private string _postID;

        public string PostID
        {
            get { return _postID; }
            set { _postID = value; }
        }

        private int _count;

        public int Count
        {
            get { return _count; }
            set { _count = value; }
        }
    }
}